// -----------------------------------------------------------------------------
#define SPON_THIS_FILE SPON Core Demo init
//
// -----------------------------------------------------------------------------
// Initialise SPON Core scripts (full debugging, default monitor interval,
// use action to open log)
//
["DEBUG_CLIENT_SERVER", nil, true] call compile preprocessFileLineNumbers "SPON\Core\initCore.sqf";

// -----------------------------------------------------------------------------
// Do some tests specific to the demo. None of this is needed outside this demo.

#include "SPON\Core\macros\macros.inc.sqf"
#include "SPON\Core\core.inc.sqf"
#include "SPON\Core\ui\handles.hpp"

#define KEY_LEFT_SQUARE_BRACKET 26
#define KEY_RIGHT_SQUARE_BRACKET 27

// -----------------------------------------------------------------------------

// Start vehicle monitoring.
call SPON_monitorVehicles;

// Tell the player what to expect from the demo.
hint ("-=| SPON Core demo |=-\n\nUse the """ + (localize "STR_SPON_DEBUG_LOG_ACTION") + """ action to bring up the " +
	"debugging log. Other than messages that appear in that log, there really " +
	"isn't anything interesting happening in this mission!\n\nPress [ for a " +
	"pointless message.\n\nPress P to raise some errors!");

// Key events...
// Init [ with up/down handlers.
_index = ["DOWN", KEY_LEFT_SQUARE_BRACKET,
	{
		SPON_GET_PARAMS_4(_keyCode,_shift,_control,_alt);
		player sideChat "[ down - Ho hum, pointless message here I come";
		SPON_TRACE_3("[ down",_shift,_control,_alt);
		
		true; // Return.
	}
] call SPON_addKeyEventHandler;

_index = ["UP", KEY_LEFT_SQUARE_BRACKET,
	{
		SPON_GET_PARAMS_4(_keyCode,_shift,_control,_alt);
		player sideChat "[ up - I told you it was a pointless message";
		SPON_TRACE_3("[ up",_shift,_control,_alt);
		
		false; // Return.
	}
] call SPON_addKeyEventHandler;

_index = ["UP", KEY_LEFT_SQUARE_BRACKET,
	{
		SPON_GET_PARAMS_4(_keyCode,_shift,_control,_alt);
		player sideChat "[ up - I told you it was a pointless message (copy)";
		SPON_TRACE_3("[ up",_shift,_control,_alt);
		
		false; // Return.
	}
] call SPON_addKeyEventHandler;

// 	
// Test events.
["testEvent1",
	{
		SPON_GET_PARAMS_1(_str);
		
		SPON_TRACE_1("Test event 1 handler 0",_str);
	}
] call SPON_addEventHandler;

_secondHandlerIndex = ["testEvent1",
	{
		SPON_GET_PARAMS_1(_str);
		
		SPON_TRACE_1("Test event 1 handler 1",_str);
	}
] call SPON_addEventHandler;

["testEvent1",
	{
		SPON_GET_PARAMS_1(_str);
		
		SPON_TRACE_1("Test event 1 handler 2",_str);
	}
] call SPON_addEventHandler;

["testEvent1",
	{
		SPON_GET_PARAMS_1(_str);
		
		SPON_TRACE_1("Test event 1 handler 3",_str);
	}
] call SPON_addEventHandler;

// Remove the second handler.
["testEvent1", _secondHandlerIndex] call SPON_removeEventHandler;

["testEvent2",
	{
		SPON_GET_PARAMS_3(_str1,_str2,_str3);
		
		SPON_TRACE_3("Test event 2 handler",_str1,_str2,_str3);
	}
] call SPON_addEventHandler;

// Test object-monitoring events.
["SPON_manFound",
	{
		SPON_GET_PARAMS_1(_man);
		
		SPON_TRACE_2("SPON_manFound handler",_man,typeOf _man);
	}
] call SPON_addEventHandler;

["SPON_vehicleFound",
	{
		SPON_GET_PARAMS_1(_vehicle);
		
		SPON_TRACE_2("SPON_vehicleFound handler",_vehicle,typeOf _vehicle);
	}
] call SPON_addEventHandler;

// Test (public) player-monitoring events.
["SPON_playerConnected",
	{
		SPON_GET_PARAMS_2(_man,_spawnedIn);
		
		SPON_TRACE_2("SPON_playerConnected handler",_man,_spawnedIn);
	}
] call SPON_addEventHandler;

["SPON_playerDisconnected",
	{
		SPON_GET_PARAMS_2(_oldMan,_oldPlayerName);
		
		SPON_TRACE_2("SPON_playerDisconnected handler",_oldMan,_oldPlayerName);
	}
] call SPON_addEventHandler;

// // Test the (private) events published when server is told about
// // arrival/departure of players.
// ["SPON_connection",
// 	{
// 		SPON_GET_PARAMS_2(_name,_id);
// 		
// 		SPON_TRACE_3("SPON_pConn handler",_name,_id,_JIP);
// 	}
// ] call SPON_addEventHandler;

// ["SPON_disconnection",
// 	{
// 		SPON_GET_PARAMS_2(_name,_id);
// 		
// 		SPON_TRACE_2("SPON_pDisconn handler",_name,_id);
// 	}
// ] call SPON_addEventHandler;

// ["SPON_updateJIP",
// 	{
// 		SPON_GET_PARAMS_3(_name,_numPlayers,_playersData);
// 		
// 		SPON_TRACE_3("SPON_pDisconn handler",_name,_numPlayers,_playersData);
// 	}
// ] call SPON_addEventHandler;
	
// Raise some local events only from the clients.
if (SPON_isClient) then
{
	SPON_WAIT_FOR_PLAYER_SYNC();

	SPON_TRACE("Raising local events...");
	["testEvent1", [str player]] call SPON_publishLocalEvent;
	["testEvent2", [str player, "Monkeys", "Hands"]] call SPON_publishLocalEvent;
};

// Raise some global events from all machines.
SPON_TRACE("Raising global events...(may take a while for these global events to be queued if published on the MP server)");
["testEvent1", [str player]] call SPON_publishGlobalEvent;
["testEvent2", [str player, "Fleas", "Knees"]] call SPON_publishGlobalEvent;
		
// Continually log messages on clients and server, just to show they are alive.
[] spawn
{
	for "_i" from 0 to 10000 do
	{
		if (SPON_isDedicatedServer) then
		{
			SPON_TRACE_1("Something happened (SERVER)",_i);
		}
		else
		{
			if (alive player) then
			{
				SPON_TRACE_2("Something happened (CLIENT)",name player,_i);
			}
			else
			{
				SPON_TRACE_1("Something happened (CLIENT) *corpse*",_i);
			};
		};
		
		sleep 5;
	};
};

// Test raising of errors.
SPON_ERROR("This error sent before the player can see the error window, " +
 "but it will still be shown when it can be seen.");

// Send the rest of the errors later, when they can be shown immediately.
["DOWN", 25, {
	SPON_GET_PARAMS_4(_keyCode,_shift,_control,_alt);
	
	hint "SPON Assertions and errors - unit tests. This should raise exactly 5 error dialogs!";
	
	SPON_ERROR("General error message. Less than exciting...(This is the last one. Really!)",
		"Some sort of error");
		
	// Run some assertions that will work.
	private ["_testCondition", "_testValue"];
	_testCondition = true;
	SPON_ASSERT(_testCondition, "THIS SHOULD NEVER HAPPEN");
	
	_testCondition = false;
	SPON_ASSERT_FALSE(_testCondition,"THIS SHOULD NEVER HAPPEN");
	SPON_ASSERT_DEFINED("_testCondition","THIS SHOULD NEVER HAPPEN");
	
	_testValue = 10;
	SPON_ASSERT_OP(_testValue,==,10,"THIS SHOULD NEVER HAPPEN");
	
	// Make assertions fail.
	_testCondition = false;
	SPON_ASSERT(_testCondition, "Testing SPON_ASSERT(). Are you still having fun? Because this never ends. Never ever. Never!!!\n\nNEVER!");
	
	_testCondition = true;
	SPON_ASSERT_FALSE(_testCondition,"Testing SPON_ASSERT_FALSE().\nSome more assertion testing.\nI know you love it.");
	
	SPON_ASSERT_DEFINED("_anUndefinedVariable","Testing SPON_ASSERT_DEFINED().");
	
	_testValue = 4;
	SPON_ASSERT_OP(_testValue,==,10,"Testing SPON_ASSERT_OP(). There is more to come. Do not worry!");

	true; // Return.
}] call SPON_addKeyEventHandler;

// BUG: This key, only when pressed down, does not work!
["DOWN", KEY_RIGHT_SQUARE_BRACKET, { hint "] down"; true }] call SPON_addKeyEventHandler;
["UP", KEY_RIGHT_SQUARE_BRACKET, { hint "] up"; true }] call SPON_addKeyEventHandler;

// UNIT TESTS (initStrings.sqf - SPON_joinStrings)
SPON_ASSERT_DEFINED("SPON_joinStrings","");

_str = [[], "x"] call SPON_joinStrings;
SPON_ASSERT_OP(_str,==,"","SPON_joinStrings 1");

_str = [[""], "x"] call SPON_joinStrings;
SPON_ASSERT_OP(_str,==,"","SPON_joinStrings 2");

_str = [["", ""], "x"] call SPON_joinStrings;
SPON_ASSERT_OP(_str,==,"x","SPON_joinStrings 3");

_str = [["a","b","c"], "x"] call SPON_joinStrings;
SPON_ASSERT_OP(_str,==,"axbxc","SPON_joinStrings 4");

// UNIT TESTS (initStrings.sqf - SPON_splitString)
SPON_ASSERT_DEFINED("SPON_splitString","");

_array = ["", "\"] call SPON_splitString;
SPON_ASSERT_OP(count _array,==,0,"SPON_splitString 1");

_array = ["\", "\"] call SPON_splitString;
SPON_TRACE_1("SPON_splitString 2",_array);
SPON_ASSERT_OP(count _array,==,2,"SPON_splitString 2");
SPON_ASSERT_OP(_array select 0,==,"","SPON_splitString 2");
SPON_ASSERT_OP(_array select 1,==,"","SPON_splitString 2");

_array = ["\frog", "\"] call SPON_splitString;
SPON_TRACE_1("SPON_splitString 3",_array);
SPON_ASSERT_OP(count _array,==,2,"SPON_splitString 3");
SPON_ASSERT_OP(_array select 0,==,"","SPON_splitString 3");
SPON_ASSERT_OP(_array select 1,==,"frog","SPON_splitString 3");

_array = ["\frog\", "\"] call SPON_splitString;
SPON_TRACE_1("SPON_splitString 4",_array);
SPON_ASSERT_OP(count _array,==,3,"SPON_splitString 4");
SPON_ASSERT_OP(_array select 0,==,"","SPON_splitString 4");
SPON_ASSERT_OP(_array select 1,==,"frog","SPON_splitString 4");
SPON_ASSERT_OP(_array select 2,==,"","SPON_splitString 4");

_array = ["cheese\frog\fish", "\"] call SPON_splitString;
SPON_TRACE_1("SPON_splitString 5",_array);
SPON_ASSERT_OP(count _array,==,3,"SPON_splitString 5");
SPON_ASSERT_OP(_array select 0,==,"cheese","SPON_splitString 5");
SPON_ASSERT_OP(_array select 1,==,"frog","SPON_splitString 5");
SPON_ASSERT_OP(_array select 2,==,"fish","SPON_splitString 5");

// UNIT TESTS (initStrings.sqf - SPON_compareStrings)
_result = ["", ""] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,0,"SPON_compareStrings 1");

_result = ["", "a"] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,-1,"SPON_compareStrings 2");

_result = ["a", ""] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,+1,"SPON_compareStrings 3");

_result = ["a", "a"] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,0,"SPON_compareStrings 4");

_result = ["a", "b"] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,-1,"SPON_compareStrings 5");

_result = ["b", "a"] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,+1,"SPON_compareStrings 6");

_result = ["aardvark", "aardwolf"] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,-1,"SPON_compareStrings 7");

_result = ["aardwolf", "aardvark"] call SPON_compareStrings;
SPON_ASSERT_OP(_result,==,+1,"SPON_compareStrings 8");

// UNIT TESTS (initSort.sqf - SPON_sortStrings)
// SPON_ASSERT_DEFINED("SPON_sortStrings","");

// _array = [[]] call SPON_sortStrings;
// SPON_ASSERT_OP(count _array,==,0,"SPON_sortStrings 1");
// SPON_ASSERT_OP(_array select 0,==,"cheese","SPON_sortStrings 1");
// SPON_ASSERT_OP(_array select 1,==,"frog","SPON_sortStrings 1");
// SPON_ASSERT_OP(_array select 2,==,"fish","SPON_sortStrings 1");

// _array = [[""]] call SPON_sortStrings;
// SPON_ASSERT_OP(count _array,==,1,"SPON_sortStrings 2");
// SPON_ASSERT_OP(_array select 0,==,"cheese","SPON_sortStrings 2");
// SPON_ASSERT_OP(_array select 1,==,"frog","SPON_sortStrings 2");
// SPON_ASSERT_OP(_array select 2,==,"fish","SPON_sortStrings 2");

// _array = [["abc", "abc"]] call SPON_sortStrings;
// SPON_ASSERT_OP(count _array,==,0,"SPON_sortStrings 3");
// SPON_ASSERT_OP(_array select 0,==,"abc","SPON_sortStrings 3");
// SPON_ASSERT_OP(_array select 1,==,"abc","SPON_sortStrings 3");

// _array = [["abc", "abcd"]] call SPON_sortStrings;
// SPON_ASSERT_OP(count _array,==,0,"SPON_sortStrings 4");
// SPON_ASSERT_OP(_array select 0,==,"abc","SPON_sortStrings 4");
// SPON_ASSERT_OP(_array select 1,==,"abcd","SPON_sortStrings 4");

// _array = [["abcd", "abc"]] call SPON_sortStrings;
// SPON_ASSERT_OP(count _array,==,0,"SPON_sortStrings 5");
// SPON_ASSERT_OP(_array select 0,==,"abc","SPON_sortStrings 5");
// SPON_ASSERT_OP(_array select 1,==,"abcd","SPON_sortStrings 5");

nil; // Return.